"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var storage_1 = require("../../libs/storage");
var dev_1 = require("../../libs/dev");
exports.TYPE_PERSIST = 'ReduxModel/Persist';
var globalStore;
var defaultPersistOption = {
    __persist: { version: -1 },
};
var KEY_PREFIX = 'ReduxModel:Persist:';
var whiteList = [];
var blackList = [];
var subscription = [];
var persistReducers = {};
var objectStrings = {};
var config = false;
var ready = false;
var readyEvents = [];
var timer;
var resetPersist = function () {
    persistReducers = {};
    restorePersist();
};
var persistIsReady = function () {
    ready = true;
    if (readyEvents.length) {
        readyEvents.forEach(function (item) { return item(); });
        readyEvents = [];
    }
};
var restorePersist = function () {
    if (!config) {
        return;
    }
    var finalReducers = {};
    Object.keys(persistReducers).forEach(function (key) {
        finalReducers[key] = objectStrings[key];
    });
    storage_1.setStorageItem(KEY_PREFIX + config.key, JSON.stringify(tslib_1.__assign(tslib_1.__assign({}, finalReducers), defaultPersistOption)));
};
var parseStorageData = function (data) {
    var _a;
    if (!config) {
        return;
    }
    if (data === null) {
        resetPersist();
    }
    else {
        try {
            var tempReducers_1 = JSON.parse(data);
            if (tempReducers_1.__persist.version === config.version) {
                var shouldRestore_1 = false;
                delete tempReducers_1.__persist;
                persistReducers = {};
                Object.keys(tempReducers_1).forEach(function (key) {
                    if (passPersistReducerName(key)) {
                        objectStrings[key] = tempReducers_1[key];
                        persistReducers[key] = JSON.parse(tempReducers_1[key]);
                    }
                    else {
                        shouldRestore_1 = true;
                    }
                });
                if (shouldRestore_1) {
                    restorePersist();
                }
            }
            else {
                resetPersist();
            }
        }
        catch (e) {
            resetPersist();
            console.error('Unable to parser persist reducers from storage: ' + e.message);
        }
    }
    if (subscription.length) {
        var payload_1 = {};
        subscription.forEach(function (key) {
            if (persistReducers.hasOwnProperty(key)) {
                payload_1[key] = persistReducers[key];
            }
        });
        subscription = [];
        (_a = globalStore) === null || _a === void 0 ? void 0 : _a.dispatch({
            type: exports.TYPE_PERSIST,
            payload: payload_1,
        });
    }
    persistIsReady();
};
exports.setPersistConfig = function (persist) {
    config = persist;
    if (persist) {
        storage_1.setStorage(persist.storage);
        whiteList = persist.whitelist ? persist.whitelist.map(function (item) { return item.getReducerName(); }) : [];
        blackList = persist.blacklist ? persist.blacklist.map(function (item) { return item.getReducerName(); }) : [];
        defaultPersistOption.__persist.version = persist.version;
    }
};
exports.handlePersist = function (store) {
    globalStore = store;
    if (config) {
        var storageData = storage_1.getStorageItem(KEY_PREFIX + config.key);
        if (storageData === null || typeof storageData === 'string') {
            parseStorageData(storageData);
        }
        else {
            storageData.then(parseStorageData);
        }
    }
    else {
        persistIsReady();
    }
};
var passPersistReducerName = function (reducerName) {
    if (!config) {
        return false;
    }
    if (blackList.length) {
        return !blackList.includes(reducerName);
    }
    else if (whiteList.length) {
        return whiteList.includes(reducerName);
    }
    return true;
};
exports.switchInitData = function (reducerName, state) {
    if (!passPersistReducerName(reducerName)) {
        return state;
    }
    // For code splitting model, we can get persist immediately.
    if (ready) {
        var persistState = persistReducers[reducerName];
        if (persistState === undefined) {
            return state;
        }
        return persistState;
    }
    subscription.push(reducerName);
    return state;
};
exports.updatePersistState = function (state) {
    if (!config || !ready) {
        return;
    }
    // Sync models + async models
    var finalReducers = tslib_1.__assign({}, persistReducers);
    var changed = false;
    Object.keys(state).forEach(function (key) {
        if (passPersistReducerName(key)) {
            finalReducers[key] = state[key];
            if (state[key] !== persistReducers[key]) {
                var tempString = JSON.stringify(state[key]);
                if (!changed && objectStrings[key] !== tempString) {
                    changed = true;
                }
                objectStrings[key] = tempString;
            }
        }
    });
    persistReducers = finalReducers;
    if (!changed) {
        return;
    }
    if (dev_1.isDebug()) {
        return restorePersist();
    }
    if (timer === undefined) {
        timer = setTimeout(function () {
            restorePersist();
            timer = undefined;
        }, 300);
    }
};
exports.onPersistReady = function (fn) {
    if (ready) {
        fn();
    }
    else {
        readyEvents.push(fn);
    }
};
exports.isPersistReady = function () {
    return ready;
};
